<?php
/**
 * Copyright © 2016 Affilae. All rights reserved.
 */
namespace Affilae\Tracking\Block;

use Magento\Framework\UrlFactory;

class BaseBlock extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Affilae\Tracking\Helper\Data
     */
    protected $_trackingHelper;

    /**
     * @var \Magento\Framework\Url
     */
    protected $_urlApp;

    /**
     * @var \Affilae\Tracking\Model\Config
     */
    protected $_config;

    /**
     * @var Magento\Sales\Model\OrderFactory
     */
    protected $_orderFactory;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * @param \Affilae\Tracking\Block\Context $context
     * @param \Magento\Framework\UrlFactory $urlFactory
     */
    public function __construct(\Affilae\Tracking\Block\Context $context)
    {
        $this->_trackingHelper = $context->getTrackingHelper();
        $this->_config         = $context->getConfig();
        $this->_urlApp         = $context->getUrlFactory()->create();
        $this->_orderFactory   = $context->getOrderFactory();
        $this->_objectManager  = $context->getObjectManager();

        parent::__construct($context);
    }

    /**
     * Function for getting the tracking helper
     *
     * @return \Affilae\Tracking\Helper\Data
     */
    public function getTrackingHelper()
    {
        return $this->_trackingHelper;
    }

    /**
     * Function for getting current url
     *
     * @return string
     */
    public function getCurrentUrl()
    {
        return $this->_urlApp->getCurrentUrl();
    }

    /**
     * Function for getting controller url for given router path
     *
     * @param string $routePath
     *
     * @return string
     */
    public function getControllerUrl($routePath)
    {
        return $this->_urlApp->getUrl($routePath);
    }

    /**
     * Function for getting current url
     *
     * @param string $path
     *
     * @return string
     */
    public function getConfigValue($path)
    {
        return $this->_config->getCurrentStoreConfigValue($path);
    }

    /**
     * Function canShowTracking
     *
     * @return bool
     */
    public function canShowTracking()
    {
        $isEnabled = $this->getConfigValue('tracking/module/is_enabled');

        if ($isEnabled) {
            $allowedIps = $this->getConfigValue('tracking/module/allowed_ip');

            if (is_null($allowedIps)) {
                return true;
            } else {
                $remoteIp = $_SERVER['REMOTE_ADDR'];

                if (strpos($allowedIps, $remoteIp) !== false) {
                    return true;
                }
            }
        }

        return false;
    }
}
