<?php
/**
 * Copyright © 2016 Affilae. All rights reserved.
 */
namespace Affilae\Tracking\Model\ResourceModel\Rule;

/**
 * Rules Collection
 */
class Collection extends \Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
{
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $_date;

    /**
     * @var int
     */
    protected $_storeId;

    /**
     * @param \Magento\Framework\Data\Collection\EntityFactory $entityFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param Magento\Store\Model\StoreManagerInterface $storeManager
     * @param null|\Zend_Db_Adapter_Abstract $connection
     * @param \Magento\Framework\Model\ResourceModel\Db\AbstractDb $resource
     */
    public function __construct(
        \Magento\Framework\Data\Collection\EntityFactory $entityFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        $connection = null,
        \Magento\Framework\Model\ResourceModel\Db\AbstractDb $resource = null
    ) {
        parent::__construct($entityFactory, $logger, $fetchStrategy, $eventManager, $connection, $resource);

        $this->_date = $date;
        $this->_storeManager = $storeManager;
    }

    /**
     * Initialize resource collection
     *
     * @return void
     */
    public function _construct()
    {
        $this->_init('Affilae\Tracking\Model\Rule', 'Affilae\Tracking\Model\ResourceModel\Rule');
        $this->_map['fields']['rule_id'] = 'main_table.rule_id';
        $this->_map['fields']['store'] = 'store_table.store_id';
        $this->_map['fields']['category'] = 'category_table.category_id';
    }

    /**
     * Perform operations after collection load
     *
     * @return $this
     */
    protected function _afterLoad()
    {
        $items = $this->getColumnValues('rule_id');
        if (count($items)) {
            $connection = $this->getConnection();
            $tableName = $this->getTable('affilae_tracking_rule_store');
            $select = $connection->select()
                ->from(['cps' => $tableName])
                ->where('cps.rule_id IN (?)', $items);

            $result = $connection->fetchPairs($select);

            if ($result) {
                foreach ($this as $item) {
                    $ruleId = $item->getData('rule_id');
                    if (!isset($result[$ruleId])) {
                        continue;
                    }
                    if ($result[$ruleId] == 0) {
                        $stores = $this->_storeManager->getStores(false, true);
                        $storeId = current($stores)->getId();
                    } else {
                        $storeId = $result[$item->getData('rule_id')];
                    }
                    $item->setData('_first_store_id', $storeId);
                    $item->setData('store_ids', [$result[$ruleId]]);
                }
            }

            if ($this->_storeId) {
                foreach ($this as $item) {
                    $item->setStoreId($this->_storeId);
                }
            }

            $map = [
                'category' => 'categories',
            ];

            foreach ($map as $key => $property) {
                $tableName = $this->getTable('affilae_tracking_rule_' . $key);
                $select = $connection->select()
                    ->from(['cps' => $tableName])
                    ->where('cps.rule_id IN (?)', $items);

                $result = $connection->fetchAll($select);
                if ($result) {
                    $data = [];
                    foreach($result as $item) {
                        $data[$item['rule_id']][] = $item[$key . '_id'];
                    }

                    foreach ($this as $item) {
                        $ruleId = $item->getData('rule_id');
                        if (isset($data[$ruleId])) {
                            $item->setData($property, $data[$ruleId]);
                        }
                    }
                }
            }
        }

        $this->_previewFlag = false;

        return parent::_afterLoad();
    }

    /**
     * Join store relation table if there is store filter
     *
     * @return void
     */
    protected function _renderFiltersBefore()
    {
        foreach(['store', 'category'] as $key) {
            if ($this->getFilter($key)) {
                $this->getSelect()->join(
                    [$key.'_table' => $this->getTable('affilae_tracking_rule_'.$key)],
                    'main_table.rule_id = '.$key.'_table.rule_id',
                    []
                )->group(
                    'main_table.rule_id'
                );
            }
        }
        parent::_renderFiltersBefore();
    }
}
